@@ -12,16 +12,13 @@ __all__ = ['IndexedComponent', 'ActiveIndexedComponent']
 
 import pyutilib.misc
 
+from pyomo.core.expr.expr_errors import TemplateExpressionError
+from pyomo.core.base.indexed_component_slice import _IndexedComponent_slice
 from pyomo.core.base.component import Component, ActiveComponent
 from pyomo.core.base.config import PyomoOptions
-from pyomo.core.base.template_expr import TemplateExpressionError
-from pyomo.util import DeveloperError
+from pyomo.common import DeveloperError
 
-from six import PY3, itervalues, iteritems, advance_iterator
-import sys
-
-import logging
-logger = logging.getLogger('pyomo.core')
+from six import PY3, itervalues, iteritems
 
 UnindexedComponent_set = set([None])
 
@@ -89,213 +86,6 @@ def _get_indexed_component_data_name(component, index):
     return ans
 
 
-class _IndexedComponent_slicer(object):
-    """Special iterator for slicing through hierarchical component trees
-
-    The basic concept is to interrupt the normal slice generation
-    procedure to return a specialized iterator (this object).  This
-    object supports simple getitem / getattr / call methods and caches
-    them until it is time to actually iterate through the slice.  We
-    then walk down the cached names / indices and resolve the final
-    objects during the iteration process.  This works because all the
-    calls to __getitem__ / __getattr__ / __call__ happen *before* the
-    first call to next()
-    """
-    attribute = 1
-    getitem = 2
-    call = 3
-
-    def __init__(self, component, fixed, sliced, ellipsis):
-        # _iter_stack holds either an iterator (if this devel in the
-        # hierarchy is a slice) or None (if this level is either a
-        # SimpleComponent or is explicitly indexed).
-        self._iter_stack = [
-            self._slice_generator(component, fixed, sliced, ellipsis) ]
-        self._call_stack = [ (0,None) ]
-        # Since this is an object, users may change these flags between
-        # where they declare the slice and iterate over it.
-        self.call_errors_generate_exceptions = True
-        self.key_errors_generate_exceptions = True
-        self.attribute_errors_generate_exceptions = True
-
-    def __iter__(self):
-        """This class implements the iterator API"""
-        return self
-
-    def next(self):
-        """__next__() iterator for Py2 compatibility"""
-        return self.__next__()
-
-    def __next__(self):
-        """Return the next element in the slice."""
-        idx = len(self._iter_stack)-1
-        while True:
-            # Flush out any non-slice levels.  Since we initialize
-            # _iter_stack with None, in the first call this will
-            # immediately walk up to the beginning of the _iter_stack
-            while self._iter_stack[idx] is None:
-                idx -= 1
-            # Get the next element in the deepest active slice
-            try:
-                _comp = advance_iterator(self._iter_stack[idx])
-                idx += 1
-            except StopIteration:
-                if not idx:
-                    # Top-level iterator is done.  We are done.
-                    # (This is how the infinite loop terminates!)
-                    raise
-                self._iter_stack[idx] = None
-                idx -= 1
-                continue
-            # Walk down the hierarchy to get to the final object
-            while idx < len(self._call_stack):
-                _call = self._call_stack[idx]
-                if _call[0] == _IndexedComponent_slicer.attribute:
-                    try:
-                        _comp = getattr(_comp, _call[1])
-                    except AttributeError:
-                        # Since we are slicing, we may only be interested in
-                        # things that match.  We will allow users to
-                        # (silently) ignore any attribute errors generated
-                        # by concrete indices in the slice hierarchy...
-                        if self.attribute_errors_generate_exceptions:
-                            raise
-                        break
-                elif _call[0] == _IndexedComponent_slicer.getitem:
-                    try:
-                        _comp = _comp.__getitem__( *(_call[1]) )
-                    except KeyError:
-                        # Since we are slicing, we may only be
-                        # interested in things that match.  We will
-                        # allow users to (silently) ignore any key
-                        # errors generated by concrete indices in the
-                        # slice hierarchy...
-                        if self.key_errors_generate_exceptions:
-                            raise
-                        break
-                    if _comp.__class__ is _IndexedComponent_slicer:
-                        # Extract the _slice_generator (for
-                        # efficiency... these are always 1-level slices,
-                        # so we don't need the overhead of the
-                        # _IndexedComponent_slicer object)
-                        self._iter_stack[idx] = _comp._iter_stack[0]
-                        try:
-                            _comp = advance_iterator(_comp)
-                        except StopIteration:
-                            # We got a slicer, but the slicer doesn't
-                            # matching anything.  We should break here,
-                            # which (due to 'while True' above) will
-                            # walk back up to the next iterator and move
-                            # on
-                            self._iter_stack[idx] = None
-                            break
-                elif _call[0] == _IndexedComponent_slicer.call:
-                    try:
-                        _comp = _comp( *(_call[1]), **(_call[2]) )
-                    except:
-                        # Since we are slicing, we may only be
-                        # interested in things that match.  We will
-                        # allow users to (silently) ignore any key
-                        # errors generated by concrete indices in the
-                        # slice hierarchy...
-                        if self.call_errors_generate_exceptions:
-                            raise
-                        break
-                idx += 1
-
-            if idx == len(self._call_stack):
-                # We have a concrete object at the end of the chain. Return it
-                return _comp
-
-    def _slice_generator(self, component, fixed, sliced, ellipsis):
-        """Utility method (generator) for generating the elements of one slice
-
-        Iterate through the component index and yield the component data
-        values that match the slice template.
-        """
-        # Handle the Ellipsis that can match any number of indices
-        explicit_index_count = len(fixed) + len(sliced)
-        if ellipsis is not None:
-            explicit_index_count = -1 - explicit_index_count
-
-        max_fixed = 0 if not fixed else max(fixed)
-
-        for index in component.__iter__():
-            # We want a tuple of indices, so convert scalars to tuples
-            _idx = index if type(index) is tuple else (index,)
-
-            # Veryfy the number of indices: if there is a wildcard
-            # slice, then there must be enough indices to at least match
-            # the fixed indices.  Without the wildcard slice, the number
-            # of indices must match exactly.
-            if explicit_index_count < 0:
-                if -explicit_index_count - 1 > len(_idx):
-                    continue
-            elif len(_idx) != explicit_index_count:
-                continue
-
-            flag = True
-            for key, val in iteritems(fixed):
-                if not val == _idx[key]:
-                    flag = False
-                    break
-            if flag:
-                # Note: it is important to use __getitem__, as the
-                # derived class may implement a non-standard storage
-                # mechanism (e.g., Param)
-                yield component[index]
-
-    def __getattr__(self, name):
-        """Override the "." operator to defer resolution until iteration.
-
-        Creating a slice of a component returns a
-        _IndexedComponent_slicer object.  Subsequent attempts to resolve
-        attributes hit this method.
-        """
-        self._iter_stack.append(None)
-        self._call_stack.append( (
-            _IndexedComponent_slicer.attribute, name ) )
-        return self
-
-    def __getitem__(self, *idx):
-        """Override the "[]" operator to defer resolution until iteration.
-
-        Creating a slice of a component returns a
-        _IndexedComponent_slicer object.  Subsequent attempts to query
-        items hit this method.
-        """
-        self._iter_stack.append(None)
-        self._call_stack.append( (
-            _IndexedComponent_slicer.getitem, idx ) )
-        return self
-
-    def __call__(self, *idx, **kwds):
-        """Special handling of the "()" operator for component slices.
-
-        Creating a slice of a component returns a _IndexedComponent_slicer
-        object.  Subsequent attempts to call items hit this method.  We
-        handle the __call__ method separately based on the item ( identifier
-        immediately before the "()") being called:
-
-        - if the item was 'component', then we defer resolution of this call
-        until we are actually iterating over the slicer.  This allows users
-        to do operations like `m.b[:].component('foo').bar[:]`
-
-        - if the item is anything else, then we will immediately iterate over
-        the slicer and call the item.  THis allows "vector-like" operations
-        like: `m.x[:,1].fix(0)`.
-        """
-        self._iter_stack.append(None)
-        self._call_stack.append( (
-            _IndexedComponent_slicer.call, idx, kwds ) )
-        if self._call_stack[-2][1] == 'component':
-            return self
-        else:
-            # Note: simply calling "list(self)" results in infinite
-            # recursion in python2.6
-            return list( i for i in self )
-
-
 class IndexedComponent(Component):
     """
     This is the base class for all indexed modeling components.
@@ -344,7 +134,8 @@ class IndexedComponent(Component):
         #
         self._data = {}
         #
-        if len(args) == 0:
+        if len(args) == 0 or (len(args) == 1 and
+                              args[0] is UnindexedComponent_set):
             #
             # If no indexing sets are provided, generate a dummy index
             #
@@ -529,7 +320,7 @@ You can silence this warning by one of three ways:
         except TypeError:
             obj = _NotFound
             index = self._processUnhashableIndex(index)
-            if index.__class__ is _IndexedComponent_slicer:
+            if index.__class__ is _IndexedComponent_slice:
                 return index
 
         if obj is _NotFound:
@@ -537,16 +328,18 @@ You can silence this warning by one of three ways:
             # due to circular imports (expr imports _VarData
             # imports indexed_component, but we need expr
             # here
-            from pyomo.core.base import expr as EXPR
-            if index.__class__ is EXPR._GetItemExpression:
+            from pyomo.core.expr import current as EXPR
+            if index.__class__ is EXPR.GetItemExpression:
                 return index
-            index = self._validate_index(index)
-            # _processUnhashableIndex could have found a slice, or
-            # _validate could have found an Ellipsis and returned a
-            # slicer
-            if index.__class__ is _IndexedComponent_slicer:
-                return index
-            obj = self._data.get(index, _NotFound)
+            validated_index = self._validate_index(index)
+            if validated_index is not index:
+                index = validated_index
+                # _processUnhashableIndex could have found a slice, or
+                # _validate could have found an Ellipsis and returned a
+                # slicer
+                if index.__class__ is _IndexedComponent_slice:
+                    return index
+                obj = self._data.get(index, _NotFound)
             #
             # Call the _getitem_when_not_present helper to retrieve/return
             # the default value
@@ -577,34 +370,34 @@ You can silence this warning by one of three ways:
             obj = _NotFound
             index = self._processUnhashableIndex(index)
 
-        if obj is not _NotFound:
-            return self._setitem_impl(index, obj, val)
-        else:
+        if obj is _NotFound:
             # If we didn't find the index in the data, then we need to
             # validate it against the underlying set (as long as
             # _processUnhashableIndex didn't return a slicer)
-            index = self._validate_index(index)
+            if index.__class__ is not _IndexedComponent_slice:
+                index = self._validate_index(index)
+        else:
+            return self._setitem_impl(index, obj, val)
         #
         # Call the _setitem_impl helper to populate the _data
         # dictionary and set the value
         #
         # Note that we need to RECHECK the class against
-        # _IndexedComponent_slicer, as _validate_index could have found
+        # _IndexedComponent_slice, as _validate_index could have found
         # an Ellipsis (which is hashable) and returned a slicer
         #
-        if index.__class__ is _IndexedComponent_slicer:
+        if index.__class__ is _IndexedComponent_slice:
             # support "m.x[:,1] = 5" through a simple recursive call.
             #
+            # Assert that this slice was just generated
+            assert len(index._call_stack) == 1
+            #
             # Note that the slicer will only slice over *existing*
             # entries, but they may not be in the data dictionary.  Make
             # a copy of the slicer items *before* we start iterating
             # over it in case the setter changes the _data dictionary.
-            for idx in list(index):
-                obj = self._data.get(idx, _NotFound)
-                if obj is _NotFound:
-                    self._setitem_when_not_present(idx, val)
-                else:
-                    self._setitem_impl(idx, obj, val)
+            for idx, obj in list(index.expanded_items()):
+                self._setitem_impl(idx, obj, val)
         else:
             obj = self._data.get(index, _NotFound)
             if obj is _NotFound:
@@ -615,6 +408,7 @@ You can silence this warning by one of three ways:
     def __delitem__(self, index):
         if self._constructed is False:
             self._not_constructed_error(index)
+
         try:
             obj = self._data.get(index, _NotFound)
         except TypeError:
@@ -622,13 +416,16 @@ You can silence this warning by one of three ways:
             index = self._processUnhashableIndex(index)
 
         if obj is _NotFound:
-            index = self._validate_index(index)
+            if index.__class__ is not _IndexedComponent_slice:
+                index = self._validate_index(index)
 
         # this supports "del m.x[:,1]" through a simple recursive call
-        if type(index) is _IndexedComponent_slicer:
+        if index.__class__ is _IndexedComponent_slice:
+            # Assert that this slice ws just generated
+            assert len(index._call_stack) == 1
             # Make a copy of the slicer items *before* we start
             # iterating over it (since we will be removing items!).
-            for idx in list(index):
+            for idx in list(index.expanded_keys()):
                 del self[idx]
         else:
             # Handle the normal deletion operation
@@ -663,7 +460,7 @@ You can silence this warning by one of three ways:
             # indexing set is a complex set operation)!
             return idx
 
-        if idx is _IndexedComponent_slicer:
+        if idx.__class__ is _IndexedComponent_slice:
             return idx
 
         if normalize_index.flatten:
@@ -671,11 +468,13 @@ You can silence this warning by one of three ways:
             # indices will be already be normalized, so we defer the
             # "automatic" call to normalize_index until now for the
             # sake of efficiency.
-            idx = normalize_index(idx)
-            if idx in self._data:
-                return idx
-            if idx in self._index:
-                return idx
+            normalized_idx = normalize_index(idx)
+            if normalized_idx is not idx:
+                idx = normalized_idx
+                if idx in self._data:
+                    return idx
+                if idx in self._index:
+                    return idx
         # There is the chance that the index contains an Ellipsis,
         # so we should generate a slicer
         if idx is Ellipsis or idx.__class__ is tuple and Ellipsis in idx:
@@ -694,7 +493,7 @@ You can silence this warning by one of three ways:
             "Index '%s' is not valid for indexed component '%s'"
             % ( idx, self.name, ))
 
-    def _processUnhashableIndex(self, idx, _exception=None):
+    def _processUnhashableIndex(self, idx):
         """Process a call to __getitem__ with unhashable elements
 
         There are three basic ways to get here:
@@ -703,6 +502,7 @@ You can silence this warning by one of three ways:
              (Simple)Component
           3) the index contains an IndexTemplate
         """
+        from pyomo.core.expr import current as EXPR
         #
         # Iterate through the index and look for slices and constant
         # components
@@ -753,48 +553,27 @@ You can silence this warning by one of three ways:
                 ellipsis = i
                 continue
 
-            if hasattr(val, 'as_numeric'):
-                _num_val = val.as_numeric()
+            if hasattr(val, 'is_expression_type'):
+                _num_val = val
                 # Attempt to retrieve the numeric value .. if this
                 # is a template expression generation, then it
                 # should raise a TemplateExpressionError
                 try:
-                    # Disable all logging for the time being.  We are
-                    # not keeping the result of this calculation - only
-                    # seeing if it is possible.  Any errors generated
-                    # evaluating the expression are not informative to
-                    # the user
-                    logging.disable(logging.CRITICAL)
-                    _num_val()
-                except TemplateExpressionError:
-                    # Not good: we have to defer this import to now
-                    # due to circular imports (expr imports _VarData
-                    # imports indexed_component, but we need expr
-                    # here
-                    from pyomo.core.base import expr as EXPR
-                    return EXPR._GetItemExpression(self, idx)
-                except:
-                    # There are other ways we could get an exception
-                    # that is not TemplateExpressionError; most notably,
-                    # evaluating a Param / Var that is not initialized.
-                    # At this point, we will silently eat that
-                    # error... it will come back again below.
-                    pass
-                finally:
-                    logging.disable(logging.NOTSET)
-
-                if _num_val.is_constant():
+                    val = EXPR.evaluate_expression(val, constant=True)
                     _found_numeric = True
-                    val = _num_val()
-                elif _num_val.is_fixed():
-                    raise RuntimeError(
-"""Error retrieving the value of an indexed item %s:
-index %s is a fixed but not constant value.  This is likely not what you
-meant to do, as if you later change the fixed value of the object this
-lookup will not change.  If you understand the implications of using
-fixed but not constant values, you can get the current value using the
-value() function.""" % ( self.name, i ))
-                else:
+
+                except TemplateExpressionError:
+                    #
+                    # The index is a template expression, so return the 
+                    # templatized expression.
+                    #
+                    from pyomo.core.expr import current as EXPR
+                    return EXPR.GetItemExpression(tuple(idx), self)
+
+                except EXPR.NonConstantExpressionError:
+                    #
+                    # The expression contains an unfixed variable
+                    #
                     raise RuntimeError(
 """Error retrieving the value of an indexed item %s:
 index %s is not a constant value.  This is likely not what you meant to
@@ -802,6 +581,24 @@ do, as if you later change the fixed value of the object this lookup
 will not change.  If you understand the implications of using
 non-constant values, you can get the current value of the object using
 the value() function.""" % ( self.name, i ))
+
+                except EXPR.FixedExpressionError:
+                    #
+                    # The expression contains a fixed variable
+                    #
+                    raise RuntimeError(
+"""Error retrieving the value of an indexed item %s:
+index %s is a fixed but not constant value.  This is likely not what you
+meant to do, as if you later change the fixed value of the object this
+lookup will not change.  If you understand the implications of using
+fixed but not constant values, you can get the current value using the
+value() function.""" % ( self.name, i ))
+                #
+                # There are other ways we could get an exception such as
+                # evaluating a Param / Var that is not initialized.
+                # These exceptions will continue up the call stack.
+                #
+
             # verify that the value is hashable
             hash(val)
             if ellipsis is None:
@@ -810,14 +607,12 @@ the value() function.""" % ( self.name, i ))
                 fixed[i - len(idx)] = val
 
         if sliced or ellipsis is not None:
-            return _IndexedComponent_slicer(self, fixed, sliced, ellipsis)
+            return _IndexedComponent_slice(self, fixed, sliced, ellipsis)
         elif _found_numeric:
             if len(idx) == 1:
                 return fixed[0]
             else:
                 return tuple( fixed[i] for i in range(len(idx)) )
-        elif _exception is not None:
-            raise
         else:
             raise DeveloperError(
                 "Unknown problem encountered when trying to retrieve "
@@ -889,6 +684,16 @@ the value() function.""" % ( self.name, i ))
                 "for scalar instances."
                 % (self.__class__.__name__,))
 
+    def _pprint(self):
+        """Print component information."""
+        return ( [("Size", len(self)),
+                  ("Index", self._index if self.is_indexed() else None),
+                  ],
+                 iteritems(self._data),
+                 ( "Object",),
+                 lambda k, v: [ type(v) ]
+                 )
+
     def id_index_map(self):
         """
         Return an dictionary id->index for
