@@ -10,14 +10,9 @@
 
 __all__ = ['pyomo_callback',
         'IPyomoExpression', 'ExpressionFactory', 'ExpressionRegistration',
-        'IModelComponent',
-        'ModelComponentFactory',
-        'register_component',
         'IPyomoPresolver', 'IPyomoPresolveAction',
-        'DataManagerFactory',
         'IParamRepresentation',
         'ParamRepresentationFactory',
-        'IModelTransformation',
         'IPyomoScriptPreprocess',
         'IPyomoScriptCreateModel',
         'IPyomoScriptCreateDataPortal',
@@ -28,19 +23,20 @@ __all__ = ['pyomo_callback',
         'IPyomoScriptPrintResults',
         'IPyomoScriptSaveResults',
         'IPyomoScriptPostprocess',
+        'ModelComponentFactory',
         'Transformation',
-        'IModelTransformation',
         'TransformationFactory',
-        'UnknownDataManager'
         ]
 
 import logging
 import pyutilib.misc
-from pyomo.util.deprecation import deprecated
-from pyomo.util.plugin import (
+from pyomo.common.deprecation import deprecated
+from pyomo.common.modeling import unique_component_name
+from pyomo.common import Factory
+from pyomo.common.plugin import (
     alias, implements, Interface, Plugin, PluginFactory, CreatePluginFactory,
     PluginError, ExtensionPoint )
-from pyomo.util.timing import TransformationTimer
+from pyomo.common.timing import TransformationTimer
 
 logger = logging.getLogger('pyomo.core')
 registered_callback = {}
@@ -178,106 +174,14 @@ def ExpressionFactory(name=None, args=[]):
 ExpressionFactory.ep = ExtensionPoint(IPyomoExpression)
 
 
-class IModelComponent(Interface):
-    pass
+class ModelComponentFactoryClass(Factory):
 
-ModelComponentFactory = CreatePluginFactory(IModelComponent)
+    def register(self, doc=None):
+        def fn(cls):
+            return super(ModelComponentFactoryClass, self).register(cls.__name__, doc)(cls)
+        return fn
 
-def register_component(cls, description):
-    class TMP(Plugin):
-        implements(IModelComponent, service=False)
-        alias(cls.__name__, description)
-        component = cls
-
-
-class IDataManager(Interface):
-
-    def available(self):
-        """ Returns True if the data manager can be executed """
-        pass
-
-    def requirements(self):
-        """ Return a string describing the packages that need to be installed for this plugin to be available """
-        pass
-
-    def initialize(self, filename, **kwds):
-        """ Prepare to read a data file. """
-        pass
-
-    def add_options(self, **kwds):
-        """ Add options """
-        pass
-
-    def open(self):
-        """ Open the data file. """
-        pass
-
-    def close(self):
-        """ Close the data file. """
-        pass
-
-    def read(self):
-        """ Read the data file. """
-        pass
-
-    def process(self, model, data, default):
-        """ Process the data. """
-        pass
-
-    def clear(self):
-        """ Reset Plugin. """
-        pass
-
-
-class UnknownDataManager(Plugin):
-
-    implements(IDataManager)
-
-    def __init__(self, *args, **kwds):
-        Plugin.__init__(self, **kwds)
-        #
-        # The 'type' is the class type of the solver instance
-        #
-        self.type = kwds["type"]
-
-    def available(self):
-        return False
-
-
-#
-# A DataManagerFactory is an instance of a plugin factory that is
-# customized with a custom __call__ method
-#
-DataManagerFactory = CreatePluginFactory(IDataManager)
-#
-# This is the custom __call__ method
-#
-def __datamanager_call__(self, _name=None, args=[], **kwds):
-    if _name is None:
-        return self
-    _name=str(_name)
-    if _name in IDataManager._factory_active:
-        dm = PluginFactory(IDataManager._factory_cls[_name], args, **kwds)
-        if not dm.available():
-            raise PluginError("Cannot process data in %s files.  The following python packages need to be installed: %s" % (_name, dm.requirements()))
-    else:
-        dm = UnknownDataManager(type=_name)
-    return dm
-#
-# Adding the the custom __call__ method to DataManagerFactory
-#
-pyutilib.misc.add_method(DataManagerFactory, __datamanager_call__, name='__call__')
-
-
-
-class IModelTransformation(Interface):
-
-    def apply(self, model, **kwds):
-        """Apply a model transformation and return a new model instance"""
-
-    def __call__(self, model, **kwds):
-        """Use this plugin instance as a functor to apply a transformation"""
-        return self.apply(model, **kwds)
+ModelComponentFactory = ModelComponentFactoryClass('model component')
 
 
 class IParamRepresentation(Interface):
@@ -301,16 +205,22 @@ class TransformationData(object):
         return self._data[name]
 
 
-class Transformation(Plugin):
+class Transformation(object):
     """
     Base class for all model transformations.
     """
-
-    implements(IModelTransformation, service=False)
-
     def __init__(self, **kwds):
         kwds["name"] = kwds.get("name", "transformation")
-        super(Transformation, self).__init__(**kwds)
+        #super(Transformation, self).__init__(**kwds)
+
+    #
+    # Support "with" statements.
+    #
+    def __enter__(self):
+        return self
+
+    def __exit__(self, t, v, traceback):
+        pass
 
     @deprecated(
         "Transformation.apply() has been deprecated.  Please use either "
@@ -350,17 +260,27 @@ class Transformation(Plugin):
             "The Transformation.apply_to method is not implemented.")
 
     def _create_using(self, model, **kwds):
+        # Put all the kwds onto the model so that when we clone the
+        # model any references to things on the model are correctly
+        # updated to point to the new instance.  Note that users &
+        # transformation developers cannot rely on things happening by
+        # argument side effect.
+        name = unique_component_name(model, '_kwds')
+        setattr(model, name, kwds)
         instance = model.clone()
+        kwds = getattr(instance, name)
+        delattr(model, name)
+        delattr(instance, name)
         self._apply_to(instance, **kwds)
         return instance
 
 
-TransformationFactory = CreatePluginFactory(IModelTransformation)
+TransformationFactory = Factory('transformation type')
 
 @deprecated()
 def apply_transformation(*args, **kwds):
     if len(args) is 0:
-        return TransformationFactory.services()
+        return list(TransformationFactory)
     xfrm = TransformationFactory(args[0])
     if len(args) == 1 or xfrm is None:
         return xfrm
