@@ -2,8 +2,8 @@
 #
 #  Pyomo: Python Optimization Modeling Objects
 #  Copyright 2017 National Technology and Engineering Solutions of Sandia, LLC
-#  Under the terms of Contract DE-NA0003525 with National Technology and 
-#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain 
+#  Under the terms of Contract DE-NA0003525 with National Technology and
+#  Engineering Solutions of Sandia, LLC, the U.S. Government retains certain
 #  rights in this software.
 #  This software is distributed under the 3-clause BSD License.
 #  ___________________________________________________________________________
@@ -13,10 +13,11 @@
 
 import pyutilib.th as unittest
 
-from pyomo.util import DeveloperError
+from pyomo.common import DeveloperError
 import pyomo.core.base._pyomo
-from pyomo.environ import *
 from pyomo.core.base.block import generate_cuid_names
+from pyomo.environ import *
+
 
 class TestComponent(unittest.TestCase):
 
