@@ -24,7 +24,8 @@ import pyutilib.th as unittest
 
 from pyomo.environ import ConcreteModel, AbstractModel, Var, Constraint, \
     ConstraintList, Param, RangeSet, Set, Expression, value, \
-    simple_constraintlist_rule, simple_constraint_rule
+    simple_constraintlist_rule, simple_constraint_rule, inequality
+from pyomo.core.expr import current as EXPR
 from pyomo.core.base.constraint import _GeneralConstraintData
 
 from six import StringIO
@@ -365,17 +366,17 @@ class TestConstraintCreation(unittest.TestCase):
         model.e2 = Expression()
         model.e3 = Expression()
         with self.assertRaises(ValueError):
-            model.c.set_value(model.e1 <= model.e2 <= model.e3)
+            model.c.set_value((model.e1, model.e2, model.e3))
         model.e1.expr = 1.0
         model.e2.expr = 1.0
         model.e3.expr = 1.0
         with self.assertRaises(ValueError):
-            model.c.set_value(model.e1 <= model.e2 <= model.e3)
+            model.c.set_value((model.e1, model.e2, model.e3))
         model.p1 = Param(mutable=True)
         model.p2 = Param(mutable=True)
-        model.c.set_value(model.p1 <= model.e1 <= model.p2)
+        model.c.set_value((model.p1, model.e1, model.p2))
         model.e1.expr = None
-        model.c.set_value(model.p1 <= model.e1 <= model.p2)
+        model.c.set_value((model.p1, model.e1, model.p2))
 
     # make sure we can use a mutable param that
     # has not been given a value in the upper bound
@@ -407,7 +408,7 @@ class TestConstraintCreation(unittest.TestCase):
         self.assertEqual(model.c.equality, False)
         model.del_component(model.c)
 
-        model.c = Constraint(expr=model.p <= model.x <= model.p + 1)
+        model.c = Constraint(expr=(model.p, model.x, model.p + 1))
         self.assertEqual(model.c.equality, False)
         model.del_component(model.c)
 
@@ -432,10 +433,6 @@ class TestConstraintCreation(unittest.TestCase):
         self.assertEqual(model.c.equality, False)
         model.del_component(model.c)
 
-        model.c = Constraint(expr=model.p + 1 >= model.x >= model.p)
-        self.assertEqual(model.c.equality, False)
-        model.del_component(model.c)
-
         model.c = Constraint(expr=(model.p, model.x, None))
         self.assertTrue(model.c.lower is model.p)
         self.assertEqual(model.c.equality, False)
@@ -483,7 +480,7 @@ class TestConstraintCreation(unittest.TestCase):
         self.assertEqual(model.c.equality, False)
         model.del_component(model.c)
 
-        model.c = Constraint(expr=model.p + 1 <= model.x <= model.p)
+        model.c = Constraint(expr=(model.p + 1, model.x, model.p))
         self.assertEqual(model.c.equality, False)
         model.del_component(model.c)
 
@@ -508,10 +505,6 @@ class TestConstraintCreation(unittest.TestCase):
         self.assertEqual(model.c.equality, False)
         model.del_component(model.c)
 
-        model.c = Constraint(expr=model.p >= model.x >= model.p + 1)
-        self.assertEqual(model.c.equality, False)
-        model.del_component(model.c)
-
         model.c = Constraint(expr=(None, model.x, model.p))
         self.assertTrue(model.c.upper is model.p)
         self.assertEqual(model.c.equality, False)
@@ -559,7 +552,7 @@ class TestConstraintCreation(unittest.TestCase):
         self.assertEqual(model.c.equality, True)
         model.del_component(model.c)
 
-        model.c = Constraint(expr=model.p <= model.x <= model.p)
+        model.c = Constraint(expr=inequality(model.p, model.x,  model.p))
         self.assertTrue(model.c.upper is model.p)
         # GH: Not sure if we are supposed to detect equality
         #     in this situation. I would rather us not, for
@@ -627,7 +620,7 @@ class TestSimpleCon(unittest.TestCase):
         model = ConcreteModel()
         model.A = RangeSet(1,4)
         model.x = Var(model.A,initialize=2)
-        model.c = Constraint(expr=0 <= sum(model.x[i] for i in model.A) <= 1)
+        model.c = Constraint(expr=(0, sum(model.x[i] for i in model.A), 1))
 
         self.assertEqual(model.c(), 8)
         self.assertEqual(value(model.c.body), 8)
@@ -657,7 +650,7 @@ class TestSimpleCon(unittest.TestCase):
             ans=0
             for i in model.B:
                 ans = ans + model.x[i]
-            return (0,ans,1)
+            return (0, ans, 1)
         model.x = Var(model.B, initialize=2)
         model.c = Constraint(rule=f)
 
@@ -672,7 +665,7 @@ class TestSimpleCon(unittest.TestCase):
             ans=0
             for i in model.B:
                 ans = ans + model.x[i]
-            return (0,ans,None)
+            return (0, ans, None)
         model.x = Var(model.B, initialize=2)
         model.c = Constraint(rule=f)
 
@@ -687,7 +680,7 @@ class TestSimpleCon(unittest.TestCase):
             ans=0
             for i in model.B:
                 ans = ans + model.x[i]
-            return (None,ans,1)
+            return (None, ans, 1)
         model.x = Var(model.B, initialize=2)
         model.c = Constraint(rule=f)
 
@@ -702,7 +695,7 @@ class TestSimpleCon(unittest.TestCase):
             ans=0
             for i in model.B:
                 ans = ans + model.x[i]
-            return (ans,1)
+            return (ans, 1)
         model.x = Var(model.B, initialize=2)
         model.c = Constraint(rule=f)
 
@@ -1140,7 +1133,7 @@ class MiscConTests(unittest.TestCase):
         self.assertEqual(model.cU.lslack(), float('inf'))
         self.assertEqual(model.cU.uslack(), 1.0)
         self.assertEqual(model.cU.slack(), 1.0)
-        model.cR = Constraint(expr=L <= model.x**2 <= U)
+        model.cR = Constraint(expr=(L, model.x**2, U))
         self.assertEqual(model.cR.lslack(), 5.0)
         self.assertEqual(model.cR.uslack(), 1.0)
         self.assertEqual(model.cR.slack(), 1.0)
@@ -1214,7 +1207,7 @@ class MiscConTests(unittest.TestCase):
             pass
         x = Var(initialize=1.0)
         x.construct()
-        a.set_value(2 >= x >= 0)
+        a.set_value((0, x, 2))
         self.assertEqual(len(a), 1)
         self.assertEqual(a(), 1)
         self.assertEqual(a.body(), 1)
@@ -1266,7 +1259,7 @@ class MiscConTests(unittest.TestCase):
         x = Var(initialize=1.0)
         x.construct()
         a.construct()
-        a.set_value(2 >= x >= 0)
+        a.set_value((0, x, 2))
         self.assertEqual(len(a), 1)
         self.assertEqual(a(), 1)
         self.assertEqual(a.body(), 1)
@@ -1313,6 +1306,7 @@ class MiscConTests(unittest.TestCase):
         except ValueError:
             pass
 
+    @unittest.skipIf(not EXPR._using_chained_inequality, "Chained inequalities are not supported.")
     def test_chainedInequalityError(self):
         m = ConcreteModel()
         m.x = Var()
@@ -1377,7 +1371,7 @@ class MiscConTests(unittest.TestCase):
         model.y = Var()
         model.z = Var()
         model.o = Constraint(rule=rule1)
-        self.assertRaises(ValueError, model.create_instance)
+        #self.assertRaises(ValueError, model.create_instance)
         #
         def rule1(model):
             expr = model.x >= model.z
@@ -1388,7 +1382,7 @@ class MiscConTests(unittest.TestCase):
         model.y = Var()
         model.z = Var()
         model.o = Constraint(rule=rule1)
-        self.assertRaises(ValueError, model.create_instance)
+        #self.assertRaises(ValueError, model.create_instance)
         #
         def rule1(model):
             expr = model.y <= model.x
@@ -1398,7 +1392,7 @@ class MiscConTests(unittest.TestCase):
         model.x = Var()
         model.y = Var()
         model.o = Constraint(rule=rule1)
-        self.assertRaises(ValueError, model.create_instance)
+        #self.assertRaises(ValueError, model.create_instance)
         #
         def rule1(model):
             expr = model.x >= model.L
@@ -1450,7 +1444,7 @@ class MiscConTests(unittest.TestCase):
         model.y = Var()
         model.z = Var()
         model.o = Constraint(rule=rule1)
-        self.assertRaises(ValueError, model.create_instance)
+        #self.assertRaises(ValueError, model.create_instance)
         #
         def rule1(model):
             expr = model.x <= model.z
@@ -1461,7 +1455,15 @@ class MiscConTests(unittest.TestCase):
         model.y = Var()
         model.z = Var()
         model.o = Constraint(rule=rule1)
-        self.assertRaises(ValueError, model.create_instance)
+        #self.assertRaises(ValueError, model.create_instance)
+        #
+        def rule1(model):
+            expr = model.x <= model.L
+            return expr
+        model = ConcreteModel()
+        model.x = Var()
+        model.L = Param(initialize=0)
+        model.o = Constraint(rule=rule1)
         #
         def rule1(model):
             expr = model.y >= model.x
@@ -1471,16 +1473,7 @@ class MiscConTests(unittest.TestCase):
         model.x = Var()
         model.y = Var()
         model.o = Constraint(rule=rule1)
-        self.assertRaises(ValueError, model.create_instance)
-        #
-        def rule1(model):
-            expr = model.x <= model.L
-            return expr
-        model = ConcreteModel()
-        model.x = Var()
-        model.L = Param(initialize=0)
-        model.o = Constraint(rule=rule1)
-
+        #self.assertRaises(ValueError, model.create_instance)
         #
         def rule1(model):
             expr = model.U <= model.x
