@@ -13,13 +13,14 @@ import inspect
 from six import iteritems
 from collections import namedtuple
 
-from pyomo.util.timing import ConstructionTimer
+from pyomo.common.timing import ConstructionTimer
+from pyomo.core.expr import current as EXPR
+from pyomo.core.expr.numvalue import ZeroConstant, _sub, native_numeric_types, as_numeric
 from pyomo.core import *
-from pyomo.core.base.plugin import register_component
+from pyomo.core.base.plugin import ModelComponentFactory
 from pyomo.core.base.numvalue import ZeroConstant, _sub
 from pyomo.core.base.misc import apply_indexed_rule
 from pyomo.core.base.block import _BlockData
-import pyomo.core.base.expr as EXPR
 
 import logging
 logger = logging.getLogger('pyomo.core')
@@ -45,26 +46,23 @@ class _ComplementarityData(_BlockData):
         # the original expressions and will result in mind-boggling
         # pprint output.
         e_ = None
-        if e.__class__ is EXPR._EqualityExpression:
-            if e._args[1].is_fixed():
-                _e = (e._args[1], e._args[0])
+        if e.__class__ is EXPR.EqualityExpression:
+            if e.arg(1).__class__ in native_numeric_types or e.arg(1).is_fixed():
+                _e = (e.arg(1), e.arg(0))
             #
             # The first argument of an equality is never fixed
             #
-            #elif e._args[0].is_fixed():
-            #    _e = (e._args[0], e._args[1])
             else:
-                _e = ( ZeroConstant, e._args[0] - e._args[1])
-        elif e.__class__ is EXPR._InequalityExpression:
-            if len(e._args) == 3:
-                _e = (e._args[0], e._args[1], e._args[2])
+                _e = ( ZeroConstant, e.arg(0) - e.arg(1))
+        elif e.__class__ is EXPR.InequalityExpression:
+            if e.arg(1).__class__ in native_numeric_types or e.arg(1).is_fixed():
+                _e = (None, e.arg(0), e.arg(1))
+            elif e.arg(0).__class__ in native_numeric_types or e.arg(0).is_fixed():
+                _e = (e.arg(0), e.arg(1), None)
             else:
-                if e._args[1].is_fixed():
-                    _e = (None, e._args[0], e._args[1])
-                elif e._args[0].is_fixed():
-                    _e = (e._args[0], e._args[1], None)
-                else:
-                    _e = ( ZeroConstant, e._args[1] - e._args[0], None )
+                _e = ( ZeroConstant, e.arg(1) - e.arg(0), None )
+        elif e.__class__ is EXPR.RangedExpression:
+                _e = (e.arg(0), e.arg(1), e.arg(2))
         else:
             _e = (None, e, None)
         return _e
@@ -91,8 +89,8 @@ class _ComplementarityData(_BlockData):
         #
         #  c:   v == expression
         #
-        _e1 = self._canonical_expression(EXPR.clone_expression(self._args[0]))
-        _e2 = self._canonical_expression(EXPR.clone_expression(self._args[1]))
+        _e1 = self._canonical_expression(self._args[0])
+        _e2 = self._canonical_expression(self._args[1])
         if len(_e1) == 2:
             # Ignore _e2; _e1 is an equality constraint
             self.c = Constraint(expr=_e1)
@@ -120,9 +118,9 @@ class _ComplementarityData(_BlockData):
             self.c._type = 1
         #
         if not _e1[0] is None and not _e1[2] is None:
-            if not _e1[0].is_constant():
+            if not (_e1[0].__class__ in native_numeric_types or _e1[0].is_constant()):
                 raise RuntimeError("Cannot express a complementarity problem of the form L < v < U _|_ g(x) where L is not a constant value")
-            if not _e1[2].is_constant():
+            if not (_e1[2].__class__ in native_numeric_types or _e1[2].is_constant()):
                 raise RuntimeError("Cannot express a complementarity problem of the form L < v < U _|_ g(x) where U is not a constant value")
             self.v = Var(bounds=(_e1[0], _e1[2]))
             self.ve = Constraint(expr=self.v == _e1[1])
@@ -135,6 +133,7 @@ class _ComplementarityData(_BlockData):
             self.ve = Constraint(expr=self.v == _e1[2] - _e1[1])
 
 
+@ModelComponentFactory.register("Complementarity conditions.")
 class Complementarity(Block):
 
     Skip = (1000,)
@@ -292,6 +291,7 @@ class IndexedComplementarity(Complementarity):
         return self._data.setdefault(idx, _ComplementarityData(self))
 
 
+@ModelComponentFactory.register("A list of complementarity conditions.")
 class ComplementarityList(IndexedComplementarity):
     """
     A complementarity component that represents a list of complementarity
@@ -358,6 +358,3 @@ class ComplementarityList(IndexedComplementarity):
                 self.add(expr)
         timer.report()
 
-
-register_component(Complementarity, "Complementarity conditions.")
-register_component(ComplementarityList, "A list of complementarity conditions.")
