@@ -8,8 +8,6 @@
 #  This software is distributed under the 3-clause BSD License.
 #  ___________________________________________________________________________
 
-#import pyomo.opt.base.log_config as _config_
-import pyomo.opt.base.log_config
 import pyomo.opt.base.opt_config
 
 from pyomo.opt.base.error import *
