@@ -45,11 +45,6 @@ class Test(unittest.TestCase):
     def tearDown(self):
         pyutilib.services.TempfileManager.clear_tempfiles()
         pyutilib.services.TempfileManager.tempdir = old_tempdir
-        #
-        # Reset all options
-        #
-        #for ep in pyomo.util.plugin.ExtensionPoint(pyomo.util.plugin.IOption):
-            #ep.reset()
 
     def test3_write_nl(self):
         """ Convert from AMPL to NL """
@@ -59,12 +54,12 @@ class Test(unittest.TestCase):
             self.model.write(currdir+'test3.nl')
         except pyutilib.common.ApplicationError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("ampl") is None:
+            if not pyomo.common.registered_executable("ampl") is None:
                 self.fail("Unexpected ApplicationError - ampl is enabled but not available: '%s'" % str(err))
             return
         except pyomo.opt.ConverterError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("ampl") is None:
+            if not pyomo.common.registered_executable("ampl") is None:
                 self.fail("Unexpected ConverterError - ampl is enabled but not available: '%s'" % str(err))
             return
         self.assertFileEqualsBaseline(currdir+'test3.nl', currdir+'test3.baseline.nl', filter=filter_nl, tolerance=1e-6)
@@ -76,31 +71,31 @@ class Test(unittest.TestCase):
             self.model.write(currdir+'test3.lp')
         except pyutilib.common.ApplicationError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("glpsol") is None:
+            if not pyomo.common.registered_executable("glpsol") is None:
                 self.fail("Unexpected ApplicationError - glpsol is enabled but not available: '%s'" % str(err))
             return
         except pyomo.opt.ConverterError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("glpsol") is None:
+            if not pyomo.common.registered_executable("glpsol") is None:
                 self.fail("Unexpected ConverterError - glpsol is enabled but not available: '%s'" % str(err))
             return
         self.assertFileEqualsBaseline(currdir+'test3.lp', currdir+'test3.baseline.lp', filter=filter, tolerance=1e-6)
 
     def test3_write_mps(self):
         """ Convert from AMPL to MPS """
-        if not pyutilib.services.registered_executable("ampl"):
+        if not pyomo.common.registered_executable("ampl"):
             self.skipTest("The ampl executable is not available")
         self.model = pyomo.opt.AmplModel(currdir+'test3.mod')
         try:
             self.model.write(currdir+'test3.mps')
         except pyutilib.common.ApplicationError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("ampl") is None:
+            if not pyomo.common.registered_executable("ampl") is None:
                 self.fail("Unexpected ApplicationError - ampl is enabled but not available: '%s'" % str(err))
             return
         except pyomo.opt.ConverterError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("ampl") is None:
+            if not pyomo.common.registered_executable("ampl") is None:
                 self.fail("Unexpected ConverterError - ampl is enabled but not available: '%s'" % str(err))
             return
         self.assertFileEqualsBaseline(currdir+'test3.mps', currdir+'test3.baseline.mps', filter=filter, tolerance=1e-6)
@@ -112,12 +107,12 @@ class Test(unittest.TestCase):
             self.model.write(currdir+'test3a.nl')
         except pyutilib.common.ApplicationError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("ampl") is None:
+            if not pyomo.common.registered_executable("ampl") is None:
                 self.fail("Unexpected ApplicationError - ampl is enabled but not available: '%s'" % str(err))
             return
         except pyomo.opt.ConverterError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("ampl") is None:
+            if not pyomo.common.registered_executable("ampl") is None:
                 self.fail("Unexpected ConverterError - ampl is enabled but not available: '%s'" % str(err))
             return
         self.assertFileEqualsBaseline(currdir+'test3a.nl', currdir+'test3.baseline.nl', filter=filter_nl, tolerance=1e-6)
@@ -129,31 +124,31 @@ class Test(unittest.TestCase):
             self.model.write(currdir+'test3a.lp')
         except pyutilib.common.ApplicationError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("glpsol") is None:
+            if not pyomo.common.registered_executable("glpsol") is None:
                 self.fail("Unexpected ApplicationError - glpsol is enabled but not available: '%s'" % str(err))
             return
         except pyomo.opt.ConverterError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("glpsol") is None:
+            if not pyomo.common.registered_executable("glpsol") is None:
                 self.fail("Unexpected ConverterError - glpsol is enabled but not available: '%s'" % str(err))
             return
         self.assertFileEqualsBaseline(currdir+'test3a.lp', currdir+'test3.baseline.lp', filter=filter, tolerance=1e-6)
 
     def test3a_write_mps(self):
         """ Convert from AMPL to MPS """
-        if not pyutilib.services.registered_executable("ampl"):
+        if not pyomo.common.registered_executable("ampl"):
             self.skipTest("The ampl executable is not available")
         self.model = pyomo.opt.AmplModel(currdir+'test3a.mod', currdir+'test3a.dat')
         try:
             self.model.write(currdir+'test3a.mps')
         except pyutilib.common.ApplicationError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("ampl") is None:
+            if not pyomo.common.registered_executable("ampl") is None:
                 self.fail("Unexpected ApplicationError - ampl is enabled but not available: '%s'" % str(err))
             return
         except pyomo.opt.ConverterError:
             err = sys.exc_info()[1]
-            if not pyutilib.services.registered_executable("ampl") is None:
+            if not pyomo.common.registered_executable("ampl") is None:
                 self.fail("Unexpected ConverterError - ampl is enabled but not available: '%s'" % str(err))
             return
         self.assertFileEqualsBaseline(currdir+'test3a.mps', currdir+'test3.baseline.mps', filter=filter, tolerance=1e-6)
