@@ -22,12 +22,12 @@ from pyutilib.misc import (ArchiveReaderFactory,
                            ArchiveReader,
                            PauseGC)
 
+from pyomo.dataportal import DataPortal
 from pyomo.core import (Block,
                         IPyomoScriptModifyInstance,
-                        DataPortal,
                         AbstractModel)
 from pyomo.core.base.block import _BlockData
-from pyomo.util.plugin import ExtensionPoint
+from pyomo.common.plugin import ExtensionPoint
 from pyomo.pysp.phutils import _OLD_OUTPUT
 from pyomo.pysp.util.misc import load_external_module
 from pyomo.pysp.scenariotree.tree_structure_model import \
@@ -466,7 +466,7 @@ class ScenarioTreeInstanceFactory(object):
         if self._scenario_tree is None:
             if (not isinstance(self._scenario_tree_model,
                                (_BlockData, Block))) and \
-               (has_networkx and \
+               ((not has_networkx) or \
                 (not isinstance(self._scenario_tree_model,
                                 networkx.DiGraph))):
                 raise TypeError(
