@@ -23,33 +23,34 @@ from pyomo.core.base.var import Var
 from pyomo.core.base.constraint import Constraint
 from pyomo.core.base.sos import SOSConstraint
 from pyomo.opt import ProblemFormat
-from pyomo.repn.canonical_repn import LinearCanonicalRepn
-from pyomo.repn.compute_canonical_repn import preprocess_block_objectives \
-    as canonical_preprocess_block_objectives
-from pyomo.repn.compute_canonical_repn import preprocess_block_constraints \
-    as canonical_preprocess_block_constraints
-from pyomo.repn.compute_canonical_repn import preprocess_constraint \
-    as canonical_preprocess_constraint
-from pyomo.repn.compute_ampl_repn import preprocess_block_objectives \
-    as ampl_preprocess_block_objectives
-from pyomo.repn.compute_ampl_repn import preprocess_block_constraints \
-    as ampl_preprocess_block_constraints
-from pyomo.repn.compute_ampl_repn import preprocess_constraint \
-    as ampl_preprocess_constraint
-from pyomo.repn.ampl_repn import generate_ampl_repn
-from pyomo.repn.canonical_repn import generate_canonical_repn
-import pyomo.util
+from pyomo.solvers.plugins.solvers.persistent_solver import PersistentSolver
+from pyomo.repn.standard_repn import (preprocess_block_objectives,
+                                      preprocess_block_constraints,
+                                      preprocess_constraint_data)
+
+import pyomo.common
 from pyomo.pysp.util.config import (PySPConfigBlock,
                                     safe_declare_common_option)
 from pyomo.pysp.util.configured_object import PySPConfiguredObject
 
-from six import iteritems, itervalues
-from six.moves import xrange
+from six import itervalues
+
+def _preprocess(model, objective=True, constraints=True):
+    objective_found = False
+    if objective:
+        for block in model.block_data_objects(active=True):
+            for obj in block.component_data_objects(Objective,
+                                                    active=True,
+                                                    descend_into=False):
+                objective_found = True
+                preprocess_block_objectives(block)
+                break
+            if objective_found:
+                break
+    if constraints:
+        for block in model.block_data_objects(active=True):
+            preprocess_block_constraints(block)
 
-canonical_expression_preprocessor = \
-    pyomo.util.PyomoAPIFactory("pyomo.repn.compute_canonical_repn")
-ampl_expression_preprocessor = \
-    pyomo.util.PyomoAPIFactory("pyomo.repn.compute_ampl_repn")
 
 #
 # We only want to do the minimal amount of work to get the instance
@@ -126,17 +127,12 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
         scenario_instance = self._scenario_instances[scenario_name]
         assert scenario_instance is not None
         for block in scenario_instance.block_data_objects(active=True):
-            assert not hasattr(block, "_gen_obj_ampl_repn")
-            assert not hasattr(block, "_gen_con_ampl_repn")
-            assert not hasattr(block, "_gen_obj_canonical_repn")
-            assert not hasattr(block, "_gen_con_canonical_repn")
-            block._gen_obj_ampl_repn = False
-            block._gen_con_ampl_repn = False
-            block._gen_obj_canonical_repn = False
-            block._gen_con_canonical_repn = False
+            assert not hasattr(block, "_gen_obj_repn")
+            assert not hasattr(block, "_gen_con_repn")
+            block._gen_obj_repn = False
+            block._gen_con_repn = False
 
     def remove_scenario(self, scenario):
-
         scenario_name = scenario.name
         assert scenario_name in self._scenario_instances
         assert scenario_name not in self._scenario_to_bundle_map
@@ -144,14 +140,10 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
         scenario_instance = self._scenario_instances[scenario_name]
         assert scenario_instance is not None
         for block in scenario_instance.block_data_objects(active=True):
-            assert not block._gen_obj_ampl_repn
-            assert not block._gen_con_ampl_repn
-            assert not block._gen_obj_canonical_repn
-            assert not block._gen_con_canonical_repn
-            del block._gen_obj_ampl_repn
-            del block._gen_con_ampl_repn
-            del block._gen_obj_canonical_repn
-            del block._gen_con_canonical_repn
+            assert not block._gen_obj_repn
+            assert not block._gen_con_repn
+            del block._gen_obj_repn
+            del block._gen_con_repn
 
         del self._scenario_instances[scenario_name]
         del self._scenario_solvers[scenario_name]
@@ -364,6 +356,7 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
 
                 if force_preprocess_bundle_objective:
                     preprocess_bundle |= preprocess_bundle_objective
+
                 if force_preprocess_bundle_constraints:
                     preprocess_bundle |= preprocess_bundle_constraints
 
@@ -393,24 +386,15 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
                                 solver.remove_constraint(con)
                                 solver.add_constraint(con)
                     else:
-                        if solver.problem_format == ProblemFormat.nl:
-                            idMap = {}
-                            if preprocess_bundle & preprocess_bundle_objective:
-                                ampl_preprocess_block_objectives(bundle_ef_instance,
-                                                                 idMap=idMap)
-                            if preprocess_bundle & preprocess_bundle_constraints:
-                                ampl_preprocess_block_constraints(bundle_ef_instance,
-                                                                  idMap=idMap)
-                        else:
-                            idMap = {}
-                            if preprocess_bundle & preprocess_bundle_objective:
-                                canonical_preprocess_block_objectives(
-                                    bundle_ef_instance,
-                                    idMap=idMap)
-                            if preprocess_bundle & preprocess_bundle_constraints:
-                                canonical_preprocess_block_constraints(
-                                    bundle_ef_instance,
-                                    idMap=idMap)
+                        idMap = {}
+                        if preprocess_bundle & preprocess_bundle_objective:
+                            preprocess_block_objectives(
+                                bundle_ef_instance,
+                                idMap=idMap)
+                        if preprocess_bundle & preprocess_bundle_constraints:
+                            preprocess_block_constraints(
+                                bundle_ef_instance,
+                                idMap=idMap)
 
         end_time = time.time()
 
@@ -487,11 +471,7 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
                         print("Running full preprocessing for scenario %s "
                               "due to fixing of variables"
                               % (scenario_name))
-
-                    if solver.problem_format() == ProblemFormat.nl:
-                        ampl_expression_preprocessor({}, model=scenario_instance)
-                    else:
-                        canonical_expression_preprocessor({}, model=scenario_instance)
+                    _preprocess(scenario_instance)
                     # We've preprocessed the entire instance, no point in checking
                     # anything else
                     _cleanup()
@@ -518,10 +498,9 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
                         solver.set_objective(obj)
             else:
                 # if only the objective changed, there is minimal work to do.
-                if solver.problem_format() == ProblemFormat.nl:
-                    ampl_preprocess_block_objectives(scenario_instance)
-                else:
-                    canonical_preprocess_block_objectives(scenario_instance)
+                _preprocess(scenario_instance,
+                            objective=True,
+                            constraints=False)
 
         if (not instance_first_preprocess) and \
            ((len(instance_constraints_updated_list) > 0) or \
@@ -559,26 +538,14 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
                           "scenario %s" % (len(instance_constraints_updated_list),
                                            scenario_name))
                 idMap = {}
-                repn_name = None
-                repn_func = None
-                if solver.problem_format() == ProblemFormat.nl:
-                    repn_name = "_ampl_repn"
-                    repn_func = generate_ampl_repn
-                else:
-                    repn_name = "_canonical_repn"
-                    repn_func = generate_canonical_repn
-
                 for list_ in (instance_constraints_updated_list,
                               instance_constraints_added_list):
                     for constraint_data in list_:
-                        if isinstance(constraint_data, LinearCanonicalRepn):
+                        if getattr(constraint_data, "_linear_canonical_form", False):
                             continue
-                        block = constraint_data.parent_block()
-                        # Get/Create the ComponentMap for the repn storage
-                        if not hasattr(block, repn_name):
-                            setattr(block, repn_name, ComponentMap())
-                        getattr(block, repn_name)[constraint_data] = \
-                            repn_func(constraint_data.body, idMap=idMap)
+                        preprocess_constraint_data(scenario_instance,
+                                                   constraint_data,
+                                                   idMap=idMap)
 
         if persistent_solver_in_use:
             if not solver.has_instance():
@@ -592,10 +559,7 @@ class ScenarioTreePreprocessor(PySPConfiguredObject):
             if self.get_option("verbose"):
                 print("Running initial full preprocessing for scenario %s"
                       % (scenario_name))
-            if solver.problem_format() == ProblemFormat.nl:
-                ampl_expression_preprocessor({}, model=scenario_instance)
-            else:
-                canonical_expression_preprocessor({}, model=scenario_instance)
+            _preprocess(scenario_instance)
         _cleanup()
 
     def modify_scenario_solver_keywords(self, scenario_name, kwds):
