@@ -12,12 +12,17 @@
 # Problem Writer for BARON .bar Format Files
 #
 
-import logging
 import itertools
+import logging
+import math
+from six import iteritems, StringIO, iterkeys
+from six.moves import xrange
+from pyutilib.math import isclose
 
-import pyomo.util.plugin
 from pyomo.opt import ProblemFormat
-from pyomo.opt.base import AbstractProblemWriter
+from pyomo.opt.base import AbstractProblemWriter, WriterFactory
+from pyomo.core.expr.numvalue import is_fixed, value, native_numeric_types, native_types
+from pyomo.core.expr import current as EXPR
 from pyomo.core.base import (SortComponents,
                              SymbolMap,
                              AlphaNumericTextLabeler,
@@ -25,21 +30,124 @@ from pyomo.core.base import (SortComponents,
                              BooleanSet, Constraint,
                              IntegerSet, Objective,
                              Var, Param)
-from pyomo.core.base.numvalue import is_fixed, value, as_numeric
 from pyomo.core.base.set_types import *
 #CLH: EXPORT suffixes "constraint_types" and "branching_priorities"
 #     pass their respective information to the .bar file
 import pyomo.core.base.suffix
-from pyomo.repn import LinearCanonicalRepn
-import pyomo.core.kernel.component_suffix
-from pyomo.core.kernel.component_block import IBlockStorage
-
-
-from six import iteritems, StringIO, iterkeys
-from six.moves import xrange
+import pyomo.core.kernel.suffix
+from pyomo.core.kernel.block import IBlock
 
 logger = logging.getLogger('pyomo.core')
 
+
+#
+# A visitor pattern that creates a string for an expression
+# that is compatible with the BARON syntax.
+#
+class ToBaronVisitor(EXPR.ExpressionValueVisitor):
+
+    def __init__(self, variables, smap):
+        super(ToBaronVisitor, self).__init__()
+        self.variables = variables
+        self.smap = smap
+
+    def visit(self, node, values):
+        """ Visit nodes that have been expanded """
+        tmp = []
+        for i,val in enumerate(values):
+            arg = node._args_[i]
+
+            if arg is None:
+                tmp.append('Undefined')
+            elif arg.__class__ in native_numeric_types:
+                tmp.append(val)
+            elif arg.__class__ in native_types:
+                tmp.append("'{0}'".format(val))
+            elif arg.is_variable_type():
+                tmp.append(val)
+            elif arg.is_expression_type() and node._precedence() < arg._precedence():
+                tmp.append("({0})".format(val))
+            else:
+                tmp.append(val)
+
+        if node.__class__ is EXPR.LinearExpression:
+            for v in node.linear_vars:
+                self.variables.add(id(v))
+
+        if node.__class__ is EXPR.ProductExpression:
+            return "{0} * {1}".format(tmp[0], tmp[1])
+        elif node.__class__ is EXPR.MonomialTermExpression:
+            if tmp[0] == '-1':
+                # It seems dumb to construct a temporary
+                # NegationExpression object Should we copy the logic
+                # from that function here?
+                return EXPR.NegationExpression._to_string(
+                    EXPR.NegationExpression((None,)),
+                    [tmp[1]], None, self.smap, True)
+            else:
+                return "{0} * {1}".format(tmp[0], tmp[1])
+        elif node.__class__ is EXPR.PowExpression:
+            x,y = node.args
+            if type(x) not in native_types and not x.is_fixed() and \
+               type(y) not in native_types and not y.is_fixed():
+                # Per the BARON manual, x ^ y is allowed as long as x
+                # and y are not both variables
+                return "exp(({1}) * log({0}))".format(tmp[0], tmp[1])
+            else:
+                return "{0} ^ {1}".format(tmp[0], tmp[1])
+        elif node.__class__ is EXPR.UnaryFunctionExpression:
+            if node.name == "sqrt":
+                return "{0} ^ 0.5".format(tmp[0])
+            elif node.name == 'log10':
+                return "{0} * log({1})".format(math.log10(math.e), tmp[0])
+            elif node.name in {'exp','log'}:
+                return node._to_string(tmp, None, self.smap, True)
+            else:
+                raise RuntimeError(
+                    'The BARON .BAR format does not support the unary '
+                    'function "%s".' % (node.name,))
+        elif node.__class__ is EXPR.AbsExpression:
+            return "({0} ^ 2) ^ 0.5".format(tmp[0])
+        else:
+            return node._to_string(tmp, None, self.smap, True)
+
+    def visiting_potential_leaf(self, node):
+        """ 
+        Visiting a potential leaf.
+
+        Return True if the node is not expanded.
+        """
+        #print("ISLEAF")
+        #print(node.__class__)
+        if node is None:
+            return True, None
+
+        if node.__class__ in native_types:
+            return True, str(node)
+
+        if node.is_variable_type():
+            if node.fixed:
+                return True, str(value(node))
+            self.variables.add(id(node))
+            label = self.smap.getSymbol(node)
+            return True, label
+
+        if not node.is_expression_type():
+            return True, str(value(node))
+
+        return False, None
+
+
+def expression_to_string(expr, variables, labeler=None, smap=None):
+    if labeler is not None:
+        if smap is None:
+            smap = SymbolMap()
+        smap.default_labeler = labeler
+    visitor = ToBaronVisitor(variables, smap)
+    return visitor.dfs_postorder_stack(expr)
+
+
+
 # TODO: The to_string function is handy, but the fact that
 #       it calls .name under the hood for all components
 #       everywhere they are used will present ENORMOUS
@@ -55,11 +163,9 @@ logger = logging.getLogger('pyomo.core')
 #       but not for numbers appearing in the objective
 #       or constraints (which are written from to_string)
 
+@WriterFactory.register('bar', 'Generate the corresponding BARON BAR file.')
 class ProblemWriter_bar(AbstractProblemWriter):
 
-    #pyomo.util.plugin.alias('baron_writer')
-    pyomo.util.plugin.alias('bar', 'Generate the corresponding BARON BAR file.')
-
     def __init__(self):
 
         AbstractProblemWriter.__init__(self, ProblemFormat.bar)
@@ -91,11 +197,7 @@ class ProblemWriter_bar(AbstractProblemWriter):
                                  all_blocks_list,
                                  active_components_data_var,
                                  symbol_map,
-                                 labeler,
-                                 create_symbol_func,
-                                 create_symbols_func,
-                                 alias_symbol_func,
-                                 object_symbol_dictionary,
+                                 c_labeler,
                                  output_fixed_variable_bounds,
                                  skip_trivial_constraints,
                                  sorter):
@@ -109,25 +211,19 @@ class ProblemWriter_bar(AbstractProblemWriter):
                     if (repn.variables is None) or \
                        (len(repn.variables) == 0):
                         return True
-                elif isinstance(constraint_data, LinearCanonicalRepn):
-                    repn = constraint_data
-                    if (repn.variables is None) or \
-                       (len(repn.variables) == 0):
-                        return True
-                else:
-                    if constraint_data.body.polynomial_degree() == 0:
-                        return True
+                elif constraint_data.body.polynomial_degree() == 0:
+                    return True
             return False
 
         #
         # Check for active suffixes to export
         #
-        if isinstance(model, IBlockStorage):
-            suffix_gen = lambda b: pyomo.core.kernel.component_suffix.\
-                         export_suffix_generator(b,
-                                                 active=True,
-                                                 return_key=True,
-                                                 descend_into=False)
+        if isinstance(model, IBlock):
+            suffix_gen = lambda b: ((suf.storage_key, suf) \
+                                    for suf in pyomo.core.kernel.suffix.\
+                                    export_suffix_generator(b,
+                                                            active=True,
+                                                            descend_into=False))
         else:
             suffix_gen = lambda b: pyomo.core.base.suffix.\
                          active_export_suffix_generator(b)
@@ -161,20 +257,6 @@ class ProblemWriter_bar(AbstractProblemWriter):
 
         non_standard_eqns = r_o_eqns + c_eqns + l_eqns
 
-        # GAH 1/5/15: Substituting all non-alphanumeric characters for underscore
-        #             in labeler so this manual update should no longer be needed
-        #
-        # If the text labeler is used, correct the labels to be
-        # baron-allowed variable names
-        # Change '(' and ')' to '__'
-        # This way, for simple variable names like 'x(1_2)' --> 'x__1_2__'
-        # FIXME: 7/21/14 This may break if users give variable names
-        #        with two or more underscores together
-        #if symbolic_solver_labels:
-        #    for key,label in iteritems(object_symbol_dictionary):
-        #        label = label.replace('(','___')
-        #        object_symbol_dictionary[key] = label.replace(')','__')
-
         #
         # EQUATIONS
         #
@@ -203,7 +285,6 @@ class ProblemWriter_bar(AbstractProblemWriter):
                                                                 sort=sorter,
                                                                 descend_into=False):
 
-
                 if (not constraint_data.has_lb()) and \
                    (not constraint_data.has_ub()):
                     assert not constraint_data.equality
@@ -214,13 +295,11 @@ class ProblemWriter_bar(AbstractProblemWriter):
 
                     eqns.append(constraint_data)
 
-                    con_symbol = \
-                        create_symbol_func(symbol_map, constraint_data, labeler)
+                    con_symbol = symbol_map.createSymbol(constraint_data, c_labeler)
                     assert not con_symbol.startswith('.')
                     assert con_symbol != "c_e_FIX_ONE_VAR_CONST__"
 
-                    alias_symbol_func(symbol_map,
-                                      constraint_data,
+                    symbol_map.alias(constraint_data,
                                       alias_template % order_counter)
                     output_file.write(", "+str(con_symbol))
                     order_counter += 1
@@ -230,11 +309,10 @@ class ProblemWriter_bar(AbstractProblemWriter):
         if n_roeqns > 0:
             output_file.write('RELAXATION_ONLY_EQUATIONS ')
             for i, constraint_data in enumerate(r_o_eqns):
-                con_symbol = create_symbol_func(symbol_map, constraint_data, labeler)
+                con_symbol = symbol_map.createSymbol(constraint_data, c_labeler)
                 assert not con_symbol.startswith('.')
                 assert con_symbol != "c_e_FIX_ONE_VAR_CONST__"
-                alias_symbol_func(symbol_map,
-                                  constraint_data,
+                symbol_map.alias(constraint_data,
                                   alias_template % order_counter)
                 if i == n_roeqns-1:
                     output_file.write(str(con_symbol)+';\n\n')
@@ -245,11 +323,10 @@ class ProblemWriter_bar(AbstractProblemWriter):
         if n_ceqns > 0:
             output_file.write('CONVEX_EQUATIONS ')
             for i, constraint_data in enumerate(c_eqns):
-                con_symbol = create_symbol_func(symbol_map, constraint_data, labeler)
+                con_symbol = symbol_map.createSymbol(constraint_data, c_labeler)
                 assert not con_symbol.startswith('.')
                 assert con_symbol != "c_e_FIX_ONE_VAR_CONST__"
-                alias_symbol_func(symbol_map,
-                                  constraint_data,
+                symbol_map.alias(constraint_data,
                                   alias_template % order_counter)
                 if i == n_ceqns-1:
                     output_file.write(str(con_symbol)+';\n\n')
@@ -260,11 +337,10 @@ class ProblemWriter_bar(AbstractProblemWriter):
         if n_leqns > 0:
             output_file.write('LOCAL_EQUATIONS ')
             for i, constraint_data in enumerate(l_eqns):
-                con_symbol = create_symbol_func(symbol_map, constraint_data, labeler)
+                con_symbol = symbol_map.createSymbol(constraint_data, c_labeler)
                 assert not con_symbol.startswith('.')
                 assert con_symbol != "c_e_FIX_ONE_VAR_CONST__"
-                alias_symbol_func(symbol_map,
-                                  constraint_data,
+                symbol_map.alias(constraint_data,
                                   alias_template % order_counter)
                 if i == n_leqns-1:
                     output_file.write(str(con_symbol)+';\n\n')
@@ -280,7 +356,7 @@ class ProblemWriter_bar(AbstractProblemWriter):
         # Example: ' x[1] ' -> ' x3 '
         #FIXME: 7/18/14 CLH: This may cause mistakes if spaces in
         #                    variable names are allowed
-        if isinstance(model, IBlockStorage):
+        if isinstance(model, IBlock):
             mutable_param_gen = lambda b: \
                                 b.components(ctype=Param,
                                              descend_into=False)
@@ -299,33 +375,38 @@ class ProblemWriter_bar(AbstractProblemWriter):
                     for param_data in param_data_iter:
                         yield param_data
 
-        vstring_to_var_dict = {}
-        vstring_to_bar_dict = {}
-        pstring_to_bar_dict = {}
-        _val_template = ' %'+self._precision_string+' '
-        for block in all_blocks_list:
-            for var_data in active_components_data_var[id(block)]:
-                variable_stream = StringIO()
-                var_data.to_string(ostream=variable_stream, verbose=False)
-                variable_string = variable_stream.getvalue()
-                variable_string = ' '+variable_string+' '
-                vstring_to_var_dict[variable_string] = var_data
-                if output_fixed_variable_bounds or (not var_data.fixed):
-                    vstring_to_bar_dict[variable_string] = \
-                        ' '+object_symbol_dictionary[id(var_data)]+' '
-                else:
-                    assert var_data.value is not None
-                    vstring_to_bar_dict[variable_string] = \
-                        (_val_template % (var_data.value,))
+        if False:
+            #
+            # This was part of a merge from master that caused
+            # test failures.  But commenting this out didn't cause additional failures!?!
+            #
+            vstring_to_var_dict = {}
+            vstring_to_bar_dict = {}
+            pstring_to_bar_dict = {}
+            _val_template = ' %'+self._precision_string+' '
+            for block in all_blocks_list:
+                for var_data in active_components_data_var[id(block)]:
+                    variable_stream = StringIO()
+                    var_data.to_string(ostream=variable_stream, verbose=False)
+                    variable_string = variable_stream.getvalue()
+                    variable_string = ' '+variable_string+' '
+                    vstring_to_var_dict[variable_string] = var_data
+                    if output_fixed_variable_bounds or (not var_data.fixed):
+                        vstring_to_bar_dict[variable_string] = \
+                            ' '+object_symbol_dictionary[id(var_data)]+' '
+                    else:
+                        assert var_data.value is not None
+                        vstring_to_bar_dict[variable_string] = \
+                            (_val_template % (var_data.value,))
 
-            for param_data in mutable_param_gen(block):
-                param_stream = StringIO()
-                param_data.to_string(ostream=param_stream, verbose=False)
-                param_string = param_stream.getvalue()
+                for param_data in mutable_param_gen(block):
+                    param_stream = StringIO()
+                    param_data.to_string(ostream=param_stream, verbose=False)
+                    param_string = param_stream.getvalue()
 
-                param_string = ' '+param_string+' '
-                pstring_to_bar_dict[param_string] = \
-                    (_val_template % (param_data(),))
+                    param_string = ' '+param_string+' '
+                    pstring_to_bar_dict[param_string] = \
+                        (_val_template % (param_data(),))
 
         # Equation Definition
         string_template = '%'+self._precision_string
@@ -335,62 +416,15 @@ class ProblemWriter_bar(AbstractProblemWriter):
                                                c_eqns,
                                                l_eqns):
 
-            #########################
-            #CLH: The section below is kind of a hack-y way to use
-            #     the expr.to_string function to print
-            #     expressions. A stream is created, writen to, and
-            #     then the string is recovered and stored in
-            #     eqn_body. Then the variable names are converted
-            #     to match the variable names that are used in the
-            #     bar file.
+            variables = set()
+            #print(symbol_map.byObject.keys())
+            eqn_body = expression_to_string(constraint_data.body, variables, smap=symbol_map)
+            #print(symbol_map.byObject.keys())
+            referenced_variable_ids.update(variables)
 
-            # Fill in the body of the equation
-            body_string_buffer = StringIO()
-
-            if constraint_data.body is None:
-                continue
-            as_numeric(constraint_data.body).to_string(
-                ostream=body_string_buffer,
-                verbose=False)
-            eqn_body = body_string_buffer.getvalue()
-
-            # First, pad the equation so that if there is a
-            # variable name at the start or end of the equation,
-            # it can still be identified as padded with spaces.
-
-            # Second, change pyomo's ** to baron's ^, also with
-            # padding so that variable can always be found with
-            # space around them
-
-            # Third, add more padding around multiplication. Pyomo
-            # already has spaces between variable on variable
-            # multiplication, but not for constants on variables
-            eqn_body = ' '+eqn_body+' '
-            eqn_body = eqn_body.replace('**',' ^ ')
-            eqn_body = eqn_body.replace('*', ' * ')
-
-
-            #
-            # FIXME: The following block of code is extremely inefficient.
-            #        We are looping through every parameter and variable in
-            #        the model each time we write a constraint expression.
-            #
-            ################################################
-            vnames = [(variable_string, bar_string)
-                      for variable_string, bar_string in iteritems(vstring_to_bar_dict)
-                      if variable_string in eqn_body]
-            for variable_string, bar_string in vnames:
-                var_data = vstring_to_var_dict[variable_string]
-                if output_fixed_variable_bounds or (not var_data.fixed):
-                    referenced_variable_ids.add(id(var_data))
-                eqn_body = eqn_body.replace(variable_string, bar_string)
-            for param_string, bar_string in iteritems(pstring_to_bar_dict):
-                eqn_body = eqn_body.replace(param_string, bar_string)
-            ################################################
-
-            if len(vnames) == 0:
+            if len(variables) == 0:
                 assert not skip_trivial_constraints
-                eqn_body += "+ 0 * ONE_VAR_CONST__ "
+                eqn_body += " + 0 * ONE_VAR_CONST__ "
 
             # 7/29/14 CLH:
             #FIXME: Baron doesn't handle many of the
@@ -402,7 +436,7 @@ class ProblemWriter_bar(AbstractProblemWriter):
             #       useful.
             ##########################
 
-            con_symbol = object_symbol_dictionary[id(constraint_data)]
+            con_symbol = symbol_map.byObject[id(constraint_data)]
             output_file.write(str(con_symbol) + ': ')
 
             # Fill in the left and right hand side (constants) of
@@ -464,46 +498,23 @@ class ProblemWriter_bar(AbstractProblemWriter):
                                      % (model.name))
 
                 # create symbol
-                create_symbol_func(symbol_map, objective_data, labeler)
-                alias_symbol_func(symbol_map, objective_data, "__default_objective__")
+                symbol_map.createSymbol(objective_data, c_labeler)
+                symbol_map.alias(objective_data, "__default_objective__")
 
                 if objective_data.is_minimizing():
                     output_file.write("minimize ")
                 else:
                     output_file.write("maximize ")
 
-                #FIXME 7/18/14 See above, constraint writing
-                #              section. Will cause problems if there
-                #              are spaces in variables
-                # Similar to the constraints section above, the
-                # objective is generated from the expr.to_string
-                # function.
-                obj_stream = StringIO()
-                objective_data.expr.to_string(ostream=obj_stream, verbose=False)
+                variables = set()
+                #print(symbol_map.byObject.keys())
+                obj_string = expression_to_string(objective_data.expr, variables, smap=symbol_map)
+                #print(symbol_map.byObject.keys())
+                referenced_variable_ids.update(variables)
 
-                obj_string = ' '+obj_stream.getvalue()+' '
-                obj_string = obj_string.replace('**',' ^ ')
-                obj_string = obj_string.replace('*', ' * ')
-
-                #
-                # FIXME: The following block of code is extremely inefficient.
-                #        We are looping through every parameter and variable in
-                #        the model each time we write an expression.
-                #
-                ################################################
-                vnames = [(variable_string, bar_string)
-                          for variable_string, bar_string in iteritems(vstring_to_bar_dict)
-                          if variable_string in obj_string]
-                for variable_string, bar_string in vnames:
-                    var_data = var_data = vstring_to_var_dict[variable_string]
-                    if output_fixed_variable_bounds or (not var_data.fixed):
-                        referenced_variable_ids.add(id(var_data))
-                    obj_string = obj_string.replace(variable_string, bar_string)
-                for param_string, bar_string in iteritems(pstring_to_bar_dict):
-                    obj_string = obj_string.replace(param_string, bar_string)
-                ################################################
 
         output_file.write(obj_string+";\n\n")
+        #referenced_variable_ids.update(symbol_map.byObject.keys())
 
         return referenced_variable_ids, branching_priorities_suffixes
 
@@ -585,17 +596,15 @@ class ProblemWriter_bar(AbstractProblemWriter):
         output_file.write("}\n\n")
 
         if symbolic_solver_labels:
-            labeler = AlphaNumericTextLabeler()
+            v_labeler = AlphaNumericTextLabeler()
+            c_labeler = AlphaNumericTextLabeler()
         elif labeler is None:
-            labeler = NumericLabeler('x')
+            v_labeler = NumericLabeler('x')
+            c_labeler = NumericLabeler('c')
 
         symbol_map = SymbolMap()
-        sm_bySymbol = symbol_map.bySymbol
-
-        #cache frequently called functions
-        create_symbol_func = SymbolMap.createSymbol
-        create_symbols_func = SymbolMap.createSymbols
-        alias_symbol_func = SymbolMap.alias
+        symbol_map.default_labeler = v_labeler
+        #sm_bySymbol = symbol_map.bySymbol
 
         # Cache the list of model blocks so we don't have to call
         # model.block_data_objects() many many times, which is slow
@@ -604,12 +613,12 @@ class ProblemWriter_bar(AbstractProblemWriter):
                                                         sort=sorter,
                                                         descend_into=True))
         active_components_data_var = {}
-        for block in all_blocks_list:
-            tmp = active_components_data_var[id(block)] = \
-                  list(obj for obj in block.component_data_objects(Var,
-                                                                   sort=sorter,
-                                                                   descend_into=False))
-            create_symbols_func(symbol_map, tmp, labeler)
+        #for block in all_blocks_list:
+        #    tmp = active_components_data_var[id(block)] = \
+        #          list(obj for obj in block.component_data_objects(Var,
+        #                                                           sort=sorter,
+        #                                                           descend_into=False))
+        #    create_symbols_func(symbol_map, tmp, labeler)
 
             # GAH: Not sure this is necessary, and also it would break for
             #      non-mutable indexed params so I am commenting out for now.
@@ -618,8 +627,8 @@ class ProblemWriter_bar(AbstractProblemWriter):
                 #if not param_data.is_constant():
                 #    create_symbol_func(symbol_map, param_data, labeler)
 
-        symbol_map_variable_ids = set(symbol_map.byObject.keys())
-        object_symbol_dictionary = symbol_map.byObject
+        #symbol_map_variable_ids = set(symbol_map.byObject.keys())
+        #object_symbol_dictionary = symbol_map.byObject
 
         #
         # Go through the objectives and constraints and generate
@@ -634,11 +643,7 @@ class ProblemWriter_bar(AbstractProblemWriter):
                 all_blocks_list,
                 active_components_data_var,
                 symbol_map,
-                labeler,
-                create_symbol_func,
-                create_symbols_func,
-                alias_symbol_func,
-                object_symbol_dictionary,
+                c_labeler,
                 output_fixed_variable_bounds,
                 skip_trivial_constraints,
                 sorter)
@@ -651,128 +656,113 @@ class ProblemWriter_bar(AbstractProblemWriter):
         IntVars = []
         PosVars = []
         Vars = []
-        for block in all_blocks_list:
-            for var_data in active_components_data_var[id(block)]:
+        for vid in referenced_variable_ids:
+            name = symbol_map.byObject[vid]
+            var_data = symbol_map.bySymbol[name]()
 
-                if id(var_data) not in referenced_variable_ids:
-                    continue
-
-                if var_data.is_continuous():
-                    if var_data.has_lb() and \
-                       (self._get_bound(var_data.lb) >= 0):
-                        TypeList = PosVars
-                    else:
-                        TypeList = Vars
-                elif var_data.is_binary():
-                    TypeList = BinVars
-                elif var_data.is_integer():
-                    TypeList = IntVars
+            if var_data.is_continuous():
+                if var_data.has_lb() and \
+                   (self._get_bound(var_data.lb) >= 0):
+                    TypeList = PosVars
                 else:
-                    assert False
-
-                var_name = object_symbol_dictionary[id(var_data)]
-                #if len(var_name) > 15:
-                #    logger.warning(
-                #        "Variable symbol '%s' for variable %s exceeds maximum "
-                #        "character limit for BARON. Solver may fail"
-                #        % (var_name, var_data.name))
-
-                TypeList.append(var_name)
+                    TypeList = Vars
+            elif var_data.is_binary():
+                TypeList = BinVars
+            elif var_data.is_integer():
+                TypeList = IntVars
+            else:
+                assert False
+            TypeList.append(name)
 
         if len(BinVars) > 0:
+            BinVars.sort()
             output_file.write('BINARY_VARIABLES ')
-            for var_name in BinVars[:-1]:
-                output_file.write(str(var_name)+', ')
-            output_file.write(str(BinVars[-1])+';\n\n')
-        if len(IntVars) > 0:
-            output_file.write('INTEGER_VARIABLES ')
-            for var_name in IntVars[:-1]:
-                output_file.write(str(var_name)+', ')
-            output_file.write(str(IntVars[-1])+';\n\n')
+            output_file.write(", ".join(BinVars))
+            output_file.write(';\n\n')
 
+        if len(IntVars) > 0:
+            IntVars.sort()
+            output_file.write('INTEGER_VARIABLES ')
+            output_file.write(", ".join(IntVars))
+            output_file.write(';\n\n')
+
+        PosVars.append('ONE_VAR_CONST__')
+        PosVars.sort()
         output_file.write('POSITIVE_VARIABLES ')
-        output_file.write('ONE_VAR_CONST__')
-        for var_name in PosVars:
-            output_file.write(', '+str(var_name))
+        output_file.write(", ".join(PosVars))
         output_file.write(';\n\n')
 
         if len(Vars) > 0:
+            Vars.sort()
             output_file.write('VARIABLES ')
-            for var_name in Vars[:-1]:
-                output_file.write(str(var_name)+', ')
-            output_file.write(str(Vars[-1])+';\n\n')
+            output_file.write(", ".join(Vars))
+            output_file.write(';\n\n')
 
         #
         # LOWER_BOUNDS
         #
 
-        LowerBoundHeader = False
-        for block in all_blocks_list:
-            for var_data in active_components_data_var[id(block)]:
+        lbounds = {}
+        for vid in referenced_variable_ids:
+            name = symbol_map.byObject[vid]
+            var_data = symbol_map.bySymbol[name]()
 
-                if id(var_data) not in referenced_variable_ids:
-                    continue
-
-                if var_data.fixed:
-                    if output_fixed_variable_bounds:
-                        var_data_lb = var_data.value
-                    else:
-                        var_data_lb = None
+            if var_data.fixed:
+                if output_fixed_variable_bounds:
+                    var_data_lb = var_data.value
                 else:
                     var_data_lb = None
-                    if var_data.has_lb():
-                        var_data_lb = self._get_bound(var_data.lb)
+            else:
+                var_data_lb = None
+                if var_data.has_lb():
+                    var_data_lb = self._get_bound(var_data.lb)
 
-                if var_data_lb is not None:
-                    if LowerBoundHeader is False:
-                        output_file.write("LOWER_BOUNDS{\n")
-                        LowerBoundHeader = True
-                    name_to_output = object_symbol_dictionary[id(var_data)]
-                    lb_string_template = '%s: %'+self._precision_string+';\n'
-                    output_file.write(lb_string_template
-                                      % (name_to_output, var_data_lb))
+            if var_data_lb is not None:
+                name_to_output = symbol_map.getSymbol(var_data)
+                lb_string_template = '%s: %'+self._precision_string+';\n'
+                lbounds[name_to_output] = lb_string_template % (name_to_output, var_data_lb)
 
-        if LowerBoundHeader:
+        if len(lbounds) > 0:
+            output_file.write("LOWER_BOUNDS{\n")
+            output_file.write("".join( lbounds[key] for key in sorted(lbounds.keys()) ) )
             output_file.write("}\n\n")
+        lbounds = None
 
         #
         # UPPER_BOUNDS
         #
 
-        UpperBoundHeader = False
-        for block in all_blocks_list:
-            for var_data in active_components_data_var[id(block)]:
+        ubounds = {}
+        for vid in referenced_variable_ids:
+            name = symbol_map.byObject[vid]
+            var_data = symbol_map.bySymbol[name]()
 
-                if id(var_data) not in referenced_variable_ids:
-                    continue
-
-                if var_data.fixed:
-                    if output_fixed_variable_bounds:
-                        var_data_ub = var_data.value
-                    else:
-                        var_data_ub = None
+            if var_data.fixed:
+                if output_fixed_variable_bounds:
+                    var_data_ub = var_data.value
                 else:
                     var_data_ub = None
-                    if var_data.has_ub():
-                        var_data_ub = self._get_bound(var_data.ub)
+            else:
+                var_data_ub = None
+                if var_data.has_ub():
+                    var_data_ub = self._get_bound(var_data.ub)
 
-                if var_data_ub is not None:
-                    if UpperBoundHeader is False:
-                        output_file.write("UPPER_BOUNDS{\n")
-                        UpperBoundHeader = True
-                    name_to_output = object_symbol_dictionary[id(var_data)]
-                    ub_string_template = '%s: %'+self._precision_string+';\n'
-                    output_file.write(ub_string_template
-                                      % (name_to_output, var_data_ub))
+            if var_data_ub is not None:
+                name_to_output = symbol_map.getSymbol(var_data)
+                ub_string_template = '%s: %'+self._precision_string+';\n'
+                ubounds[name_to_output] = ub_string_template % (name_to_output, var_data_ub)
 
-        if UpperBoundHeader:
+        if len(ubounds) > 0:
+            output_file.write("UPPER_BOUNDS{\n")
+            output_file.write("".join( ubounds[key] for key in sorted(ubounds.keys()) ) )
             output_file.write("}\n\n")
+        ubounds = None
 
         #
         # BRANCHING_PRIORITIES
         #
 
-        # Specifyig priorities requires that the pyomo model has established an
+        # Specifying priorities requires that the pyomo model has established an
         # EXTERNAL, float suffix called 'branching_priorities' on the model
         # object, indexed by the relevant variable
         BranchingPriorityHeader = False
@@ -784,7 +774,7 @@ class ProblemWriter_bar(AbstractProblemWriter):
                     if not BranchingPriorityHeader:
                         output_file.write('BRANCHING_PRIORITIES{\n')
                         BranchingPriorityHeader = True
-                    name_to_output = object_symbol_dictionary[id(var_data)]
+                    name_to_output = symbol_map.getSymbol(var_data)
                     output_file.write(name_to_output+': '+str(priority)+';\n')
 
         if BranchingPriorityHeader:
@@ -799,33 +789,21 @@ class ProblemWriter_bar(AbstractProblemWriter):
         # STARTING_POINT
         #
         output_file.write('STARTING_POINT{\nONE_VAR_CONST__: 1;\n')
+        tmp = {}
         string_template = '%s: %'+self._precision_string+';\n'
-        for block in all_blocks_list:
-            for var_data in active_components_data_var[id(block)]:
+        for vid in referenced_variable_ids:
+            name = symbol_map.byObject[vid]
+            var_data = symbol_map.bySymbol[name]()
 
-                if id(var_data) not in referenced_variable_ids:
-                    continue
-
-                starting_point = var_data.value
-                if starting_point is not None:
-                    var_name = object_symbol_dictionary[id(var_data)]
-                    output_file.write(string_template % (var_name, starting_point))
+            starting_point = var_data.value
+            if starting_point is not None:
+                var_name = symbol_map.getSymbol(var_data)
+                tmp[var_name] = string_template % (var_name, starting_point)
 
+        output_file.write("".join( tmp[key] for key in sorted(tmp.keys()) ))
         output_file.write('}\n\n')
 
         output_file.close()
 
-        # Clean up the symbol map to only contain variables referenced
-        # in the active constraints
-        vars_to_delete = symbol_map_variable_ids - referenced_variable_ids
-        sm_byObject = symbol_map.byObject
-        for varid in vars_to_delete:
-            symbol = sm_byObject[varid]
-            del sm_byObject[varid]
-            del sm_bySymbol[symbol]
-
-        del symbol_map_variable_ids
-        del referenced_variable_ids
-
         return output_filename, symbol_map
 
