@@ -10,10 +10,9 @@
 
 import os
 
-import pyutilib.services
+import pyomo.common
 import pyutilib.misc
 
-import pyomo.util.plugin
 from pyomo.opt.base import *
 from pyomo.opt.base.solvers import _extract_version
 from pyomo.opt.results import *
@@ -27,12 +26,13 @@ try:
 except:
     basestring = str
 
+
+@SolverFactory.register('conopt', doc='The CONOPT NLP solver')
 class CONOPT(SystemCallSolver):
     """
     An interface to the CONOPT optimizer that uses the AMPL Solver Library.
     """
 
-    pyomo.util.plugin.alias('conopt', doc='The CONOPT NLP solver')
 
     def __init__(self, **kwds):
         #
@@ -62,7 +62,7 @@ class CONOPT(SystemCallSolver):
         return ResultsFormat.sol
 
     def _default_executable(self):
-        executable = pyutilib.services.registered_executable("conopt")
+        executable = pyomo.common.registered_executable("conopt")
         if executable is None:
             logger.warning("Could not locate the 'conopt' executable, "
                            "which is required for solver %s" % self.name)
@@ -161,4 +161,4 @@ class CONOPT(SystemCallSolver):
             results.solver.status = SolverStatus.ok
         return results
 
-pyutilib.services.register_executable(name="conopt")
+pyomo.common.register_executable(name="conopt")
