@@ -15,31 +15,30 @@ import time
 import logging
 import subprocess
 
-import pyutilib.services
+import pyomo.common
 import pyutilib.misc
 
-import pyomo.util.plugin
 from pyomo.opt.base import *
 from pyomo.opt.base.solvers import _extract_version
 from pyomo.opt.results import *
 from pyomo.opt.solver import *
-from pyomo.core.kernel.component_block import IBlockStorage
+from pyomo.core.kernel.block import IBlock
 
 logger = logging.getLogger('pyomo.solvers')
 
-from six import iteritems
+from six import iteritems, StringIO
 
 try:
     unicode
 except:
     basestring = str
 
+
+@SolverFactory.register('gurobi', doc='The GUROBI LP/MIP solver')
 class GUROBI(OptSolver):
     """The GUROBI LP/MIP solver
     """
 
-    pyomo.util.plugin.alias('gurobi', doc='The GUROBI LP/MIP solver')
-
     def __new__(cls, *args, **kwds):
         try:
             mode = kwds['solver_io']
@@ -79,12 +78,12 @@ class GUROBI(OptSolver):
         return opt
 
 
+
+@SolverFactory.register('_gurobi_shell',  doc='Shell interface to the GUROBI LP/MIP solver')
 class GUROBISHELL(ILMLicensedSystemCallSolver):
     """Shell interface to the GUROBI LP/MIP solver
     """
 
-    pyomo.util.plugin.alias('_gurobi_shell',  doc='Shell interface to the GUROBI LP/MIP solver')
-
     def __init__(self, **kwds):
         #
         # Call base class constructor
@@ -172,7 +171,7 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
         # **Note**: This assumes that the symbol_map is "clean", i.e.,
         # contains only references to the variables encountered in constraints
         output_index = 0
-        if isinstance(instance, IBlockStorage):
+        if isinstance(instance, IBlock):
             smap = getattr(instance,"._symbol_maps")\
                    [self._smap_id]
         else:
@@ -245,9 +244,9 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
 
     def _default_executable(self):
         if sys.platform == 'win32':
-            executable = pyutilib.services.registered_executable("gurobi.bat")
+            executable = pyomo.common.registered_executable("gurobi.bat")
         else:
-            executable = pyutilib.services.registered_executable("gurobi.sh")
+            executable = pyomo.common.registered_executable("gurobi.sh")
         if executable is None:
             logger.warning("Could not locate the 'gurobi' executable, "
                            "which is required for solver %s" % self.name)
@@ -262,17 +261,14 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
         solver_exec = self.executable()
         if solver_exec is None:
             return _extract_version('')
-        outname = pyutilib.services.TempfileManager.create_tempfile(suffix = '.gurobi.version')
-        with open(outname,'w') as f:
-            # **Note, adding a 'timelimit' keyword here results in empty output for some reason
-            results = pyutilib.subprocess.run([solver_exec],
-                                              stdin=('from gurobipy import *; '
-                                                     'print(gurobi.version()); exit()'),
-                                              ostream=f)
+        f = StringIO()
+        results = pyutilib.subprocess.run([solver_exec],
+                                          stdin=('from gurobipy import *; '
+                                                 'print(gurobi.version()); exit()'),
+                                          ostream=f)
         tmp = None
         try:
-            with open(outname,'r') as f:
-                tmp = tuple(eval(f.read().strip()))
+            tmp = tuple(eval(f.getvalue().strip()))
             while(len(tmp) < 4):
                 tmp += (0,)
         except SyntaxError:
@@ -541,6 +537,6 @@ class GUROBISHELL(ILMLicensedSystemCallSolver):
         return results
 
 if sys.platform == 'win32':
-    pyutilib.services.register_executable(name='gurobi.bat')
+    pyomo.common.register_executable(name='gurobi.bat')
 else:
-    pyutilib.services.register_executable(name='gurobi.sh')
+    pyomo.common.register_executable(name='gurobi.sh')
