@@ -10,10 +10,9 @@
 
 import os
 
-import pyutilib.services
+import pyomo.common
 import pyutilib.misc
 
-import pyomo.util.plugin
 from pyomo.opt.base import *
 from pyomo.opt.base.solvers import _extract_version
 from pyomo.opt.results import *
@@ -27,12 +26,12 @@ try:
 except:
     basestring = str
 
+
+@SolverFactory.register('scip', doc='The SCIP LP/MIP solver')
 class SCIPAMPL(SystemCallSolver):
     """A generic optimizer that uses the AMPL Solver Library to interface with applications.
     """
 
-    pyomo.util.plugin.alias('scip', doc='The SCIP LP/MIP solver')
-
     def __init__(self, **kwds):
         #
         # Call base constructor
@@ -61,7 +60,7 @@ class SCIPAMPL(SystemCallSolver):
         return ResultsFormat.sol
 
     def _default_executable(self):
-        executable = pyutilib.services.registered_executable("scipampl")
+        executable = pyomo.common.registered_executable("scipampl")
         if executable is None:
             logger.warning("Could not locate the 'scipampl' executable, "
                            "which is required for solver %s" % self.name)
@@ -293,4 +292,4 @@ class SCIPAMPL(SystemCallSolver):
                     SolutionStatus.unknown
 
         return results
-pyutilib.services.register_executable(name="scipampl")
+pyomo.common.register_executable(name="scipampl")
